#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import glob
from pwact.active_learning.slurm.slurm import SlurmJob, Mission, scancle_job
from pwact.utils.slurm_script import get_slurm_job_run_info, set_slurm_script_content, split_job_for_group
from pwact.active_learning.user_input.resource import Resource
from pwact.active_learning.user_input.iter_input import InputParam

from pwact.utils.format_input_output import make_train_name, get_seed_by_time, get_iter_from_iter_name, make_iter_name
from pwact.utils.constant import AL_STRUCTURE, UNCERTAINTY, TEMP_STRUCTURE, MODEL_CMD, \
    TRAIN_INPUT_PARAM, TRAIN_FILE_STRUCTUR, FORCEFILED, LABEL_FILE_STRUCTURE, SLURM_OUT, MODEL_TYPE, PWDATA, INIT_BULK

from pwact.utils.file_operation import save_json_file, write_to_file, del_dir, search_files, add_postfix_dir, mv_file, copy_dir, del_file_list, del_file_list_by_patten
'''
description: model training method:
1. go to train data path
2. set training command
3. training
4. post process
param {*} itername
param {*} work_type
return {*}
'''

class ModelTrian(object):
    @staticmethod
    def kill_job(root_dir:str, itername:str):
        train_dir =  os.path.join(root_dir, itername, TEMP_STRUCTURE.tmp_run_iter_dir, AL_STRUCTURE.train)
        scancle_job(train_dir)

    def __init__(self, itername:str, resource: Resource, input_param:InputParam):
        self.itername = itername
        self.resource = resource
        self.input_param = input_param
        self.train_flag = True
        self.iter = get_iter_from_iter_name(self.itername)
        # train work dir
        self.train_dir = os.path.join(self.input_param.root_dir, self.itername, TEMP_STRUCTURE.tmp_run_iter_dir, AL_STRUCTURE.train)
        self.real_train_dir = os.path.join(self.input_param.root_dir, self.itername, AL_STRUCTURE.train)
    
    '''
    description: 
        if the slurm jobs done
        it means this iter done before, need back up train files in iter*/train directory 
    param {*} self
    return {*}
    author: wuxingxing
    '''    
    def back_train(self):
        slurm_remain, slurm_success = get_slurm_job_run_info(self.real_train_dir, \
            job_patten="*-{}".format(TRAIN_FILE_STRUCTUR.train_job), \
            tag_patten="*-{}".format(TRAIN_FILE_STRUCTUR.train_tag),
            for_back = True)
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False # len(slurm_remain) > 0 exist slurm jobs need to do
        if slurm_done:
            # bk and do new job
            target_bk_file = add_postfix_dir(self.real_train_dir, postfix_str="bk")
            mv_file(self.real_train_dir, target_bk_file)
            # if the temp_work_dir/train exists, delete the train dir
            if os.path.exists(self.train_dir):
                del_dir(self.train_dir)
                
    def make_train_work(self):
        train_list = []
        for model_index in range(0, self.input_param.strategy.model_num):
            # make model_i work dir
            model_i = make_train_name(model_index)
            model_i_dir = os.path.join(self.train_dir, model_i)
            if not os.path.exists(model_i_dir):
                os.makedirs(model_i_dir)
            # make train.json file
            train_dict, train_tag = self.set_train_input_dict(work_dir=model_i_dir, model_index = model_index)
            train_json_file_path = os.path.join(model_i_dir, TRAIN_FILE_STRUCTUR.train_json)
            save_json_file(train_dict, train_json_file_path)
            train_list.append(model_i_dir)
        if train_tag:
            self.make_train_slurm_job_files(train_list)
        else:
            pre_iter_name = make_iter_name(self.iter - 1)
            pre_iter_dir = os.path.join(self.input_param.root_dir, pre_iter_name, AL_STRUCTURE.train)
            copy_dir(pre_iter_dir, self.train_dir)
    
    def make_train_slurm_job_files(self, train_list:list[str]):
        # make train slurm script
        del_file_list_by_patten(self.train_dir, "*{}".format(TRAIN_FILE_STRUCTUR.train_job))
        group_list = split_job_for_group(self.resource.train_resource.group_size, train_list, 1)
        for group_index, group in enumerate(group_list):
            if group[0] == "NONE":
                continue

            jobname = "train{}".format(group_index)
            tag_name = "{}-{}".format(group_index, TRAIN_FILE_STRUCTUR.train_tag)
            tag = os.path.join(self.train_dir, tag_name)
            run_cmd = self.set_train_cmd()

            group_slurm_script = set_slurm_script_content(gpu_per_node=self.resource.train_resource.gpu_per_node, 
                number_node = self.resource.train_resource.number_node, 
                cpu_per_node = self.resource.train_resource.cpu_per_node,
                queue_name = self.resource.train_resource.queue_name,
                custom_flags = self.resource.train_resource.custom_flags,
                env_script = self.resource.train_resource.env_script,
                job_name = jobname,
                run_cmd_template = run_cmd,
                group = group,
                job_tag = tag,
                task_tag = TRAIN_FILE_STRUCTUR.train_tag, 
                task_tag_faild = TRAIN_FILE_STRUCTUR.train_tag_failed,
                parallel_num=1,
                check_type=None
                )
            slurm_script_name = "{}-{}".format(group_index, TRAIN_FILE_STRUCTUR.train_job)
            slurm_job_file = os.path.join(self.train_dir, slurm_script_name)
            write_to_file(slurm_job_file, group_slurm_script, "w")

    def set_train_cmd(self):
        model_path = "./{}/{}".format(TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.dp_model_name)
        cmp_model_path = None
        script = ""
        pwmlff = self.resource.train_resource.command
        script += "{} {} {} >> {}\n\n".format(pwmlff, MODEL_CMD.train, TRAIN_FILE_STRUCTUR.train_json, SLURM_OUT.train_out)

        # do nothing for nep model
        if self.input_param.train.model_type == MODEL_TYPE.dp:
            if self.input_param.strategy.compress:
                script += "    {} {} {} -d {} -o {} -s {}/{} >> {}\n\n".format(pwmlff, MODEL_CMD.compress, model_path, \
                    self.input_param.strategy.compress_dx, self.input_param.strategy.compress_order, TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.compree_dp_name, SLURM_OUT.train_out)
                cmp_model_path = "{}/{}".format(TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.compree_dp_name)
            
            if self.input_param.strategy.md_type == FORCEFILED.libtorch_lmps:
                if self.resource.explore_resource.gpu_per_node is None or self.resource.explore_resource.gpu_per_node == 0:
                    script += "    export CUDA_VISIBLE_DEVICES=''\n"
                if cmp_model_path is None:
                    # script model_record/dp_model.ckpt the torch_script_module.pt will in model_record dir
                    script += "    {} {} {} {}/{} >> {}\n".format(pwmlff, MODEL_CMD.script, model_path, TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.script_dp_name, SLURM_OUT.train_out)
                else:
                    script += "    {} {} {} {}/{} >> {}\n\n".format(pwmlff, MODEL_CMD.script, cmp_model_path, TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.script_dp_name, SLURM_OUT.train_out)
        return script

    '''
    description: 
        if the init format exists movement files, convert them to npy format and save to root_dir/init_data dir
        then in each iter, the init_data is from the root_dir
        make train json dict content
    param {*} self
    return {*}
    author: wuxingxing
    '''
    def set_train_input_dict(self, work_dir:str=None, model_index=None):
        train_json = self.input_param.train.to_dict()
        if self.iter == 0 and len(self.input_param.init_model_list) > 0: 
            train_json[TRAIN_INPUT_PARAM.recover_train] = True
            train_json[TRAIN_INPUT_PARAM.model_load_file] = self.input_param.init_model_list[model_index]
            train_json[TRAIN_INPUT_PARAM.optimizer][TRAIN_INPUT_PARAM.reset_epoch] = True            
        if self.iter > 0 and self.input_param.use_pre_model:
            # use old model param iter.*/train/train.000/model_record/dp_model.ckpt
            pre_model = os.path.join(self.input_param.root_dir, make_iter_name(self.iter-1), \
                AL_STRUCTURE.train, make_train_name(0), TRAIN_FILE_STRUCTUR.model_record, TRAIN_FILE_STRUCTUR.dp_model_name)
            train_json[TRAIN_INPUT_PARAM.recover_train] = True
            train_json[TRAIN_INPUT_PARAM.model_load_file] = pre_model
            train_json[TRAIN_INPUT_PARAM.optimizer][TRAIN_INPUT_PARAM.reset_epoch] = True
        train_feature_path = []
        for _data in self.input_param.init_data:
            train_feature_path.append(_data)
        # search train_feature_path in iter*/label/result/*/PWdata/*
        iter_index = get_iter_from_iter_name(self.itername)
        start_iter = 0
        train_tag = True
        while start_iter < iter_index:
            if self.input_param.data_format == PWDATA.extxyz: # result/train.xyz
                iter_data_list = search_files(self.input_param.root_dir, 
                                    "{}/{}/{}/{}".format(make_iter_name(start_iter),
                                         AL_STRUCTURE.labeling, LABEL_FILE_STRUCTURE.result, INIT_BULK.get_save_format(self.input_param.data_format)))
            else:#pwmlff/npy 'iter.***/label/result/*/*' -> result/PWdata/dir
                iter_data_list = search_files(self.input_param.root_dir, 
                                    "{}/{}/{}/*/*".format(make_iter_name(start_iter),
                                         AL_STRUCTURE.labeling, LABEL_FILE_STRUCTURE.result))

            if len(iter_data_list) > 0:
                train_feature_path.extend(iter_data_list)
            start_iter += 1
        
        if start_iter > 0 and len(iter_data_list) == 0:
            train_tag = False

        # reset seed
        train_json[TRAIN_INPUT_PARAM.seed] = get_seed_by_time()
        train_json[TRAIN_INPUT_PARAM.raw_files] = []
        train_json[TRAIN_INPUT_PARAM.train_data] = train_feature_path
        # set valid data
        train_json[TRAIN_INPUT_PARAM.valid_data] = self.input_param.valid_data
        train_json[TRAIN_INPUT_PARAM.format] = self.input_param.data_format
        if self.input_param.strategy.uncertainty == UNCERTAINTY.kpu:
            train_json[TRAIN_INPUT_PARAM.save_p_matrix] = True
        return train_json, train_tag

    def do_train_job(self):
        mission = Mission()
        slurm_remain, slurm_success = get_slurm_job_run_info(self.train_dir, \
            job_patten="*-{}".format(TRAIN_FILE_STRUCTUR.train_job), \
            tag_patten="*-{}".format(TRAIN_FILE_STRUCTUR.train_tag))
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False
        if slurm_done is False:
            #recover slurm jobs
            if len(slurm_remain) > 0:
                print("Run these train Jobs:\n")
                print(slurm_remain)
                for i, script_path in enumerate(slurm_remain):
                    slurm_job = SlurmJob()
                    tag_name = "{}-{}".format(os.path.basename(script_path).split('-')[0].strip(), TRAIN_FILE_STRUCTUR.train_tag)
                    tag = os.path.join(os.path.dirname(script_path),tag_name)
                    slurm_job.set_tag(tag)
                    slurm_job.set_cmd(script_path)
                    mission.add_job(slurm_job)

            if len(mission.job_list) > 0:
                mission.commit_jobs()
                mission.check_running_job()
                mission.all_job_finished(error_type=SLURM_OUT.train_out)

    def post_process_train(self):
        copy_dir(self.train_dir, self.real_train_dir)
        del_file_list(search_files(self.real_train_dir, "slurm*.out"))
        del_file_list(search_files(self.real_train_dir, "*/{}".format(SLURM_OUT.train_out)))



import os
import glob

from pwact.active_learning.init_bulk.relax import Relax
from pwact.active_learning.init_bulk.duplicate_scale import do_pertub_work, do_post_pertub, pertub_done, \
    duplicate_scale_done, duplicate_scale, do_post_duplicate_scale
from pwact.active_learning.init_bulk.aimd import AIMD
from pwact.active_learning.init_bulk.explore import BIGMODEL
from pwact.active_learning.init_bulk.relabel import Relabel
from pwact.active_learning.user_input.init_bulk_input import InitBulkParam
from pwact.active_learning.user_input.resource import Resource
from pwact.active_learning.slurm.slurm import scancle_job
from pwact.utils.constant import INIT_BULK, DFT_STYLE, TEMP_STRUCTURE, PWDATA, LABEL_FILE_STRUCTURE
from pwact.utils.file_operation import copy_file, copy_dir, search_files, del_file, del_file_list, write_to_file, del_file_list_by_patten
from pwact.data_format.configop import extract_pwdata

def init_bulk_run(resource: Resource, input_param:InitBulkParam):
    #1. do relax
    if input_param.is_relax:
        relax = Relax(resource, input_param)
        if not relax.check_work_done():
            # make relax work dir
            relax.make_relax_work()
            # do relax jobs
            relax.do_relax_jobs()
            # do post process
        relax.do_post_process()
    # do super cell and scale
    if not duplicate_scale_done(input_param):
        duplicate_scale(resource, input_param)
        do_post_duplicate_scale(resource, input_param)

    # do pertub
    if not pertub_done(input_param):
        do_pertub_work(resource, input_param)
        do_post_pertub(resource, input_param)

    # do aimd
    if input_param.is_aimd:
        aimd = AIMD(resource, input_param)
        if not aimd.check_work_done():
            aimd.make_aimd_work()
            aimd.do_aimd_jobs()
            aimd.do_post_process()

    elif input_param.is_bigmodel:
        bigmodel = BIGMODEL(resource, input_param)
        if not bigmodel.check_work_done():
            bigmodel.make_bigmodel_work()
            bigmodel.do_bigmodel_jobs()
            bigmodel.do_post_process()
            
        # do direct
        if not bigmodel.check_direct_done():
            bigmodel.make_direct_work()
            bigmodel.do_direct_jobs() #  after 

        # do relabel
        if input_param.is_scf:
            relabel = Relabel(resource, input_param)
            if not relabel.check_work_done():
                relabel.make_scf_work()
                relabel.do_scf_jobs()
                relabel.do_post_process()
                
    do_collection(resource, input_param)
       
def do_collection(resource: Resource, input_param:InitBulkParam):
    init_configs = input_param.sys_config
    
    relax_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.relax)
    duplicate_scale_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.super_cell_scale) 
    pertub_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.pertub)
    aimd_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.aimd)
    collection_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.collection)
    bigmodel_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.bigmodel)
    direct_dir = os.path.join(bigmodel_dir, INIT_BULK.direct)
    
    real_collection_dir = os.path.join(input_param.root_dir, INIT_BULK.collection)
    real_relax_dir = os.path.join(input_param.root_dir, INIT_BULK.relax)
    real_duplicate_scale_dir = os.path.join(input_param.root_dir, INIT_BULK.super_cell_scale) 
    real_pertub_dir = os.path.join(input_param.root_dir, INIT_BULK.pertub)
    real_aimd_dir = os.path.join(input_param.root_dir, INIT_BULK.aimd)

    relabel_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.scf)
    real_relabel_dir = os.path.join(input_param.root_dir, INIT_BULK.scf)
    real_bigmodel_dir = os.path.join(input_param.root_dir, INIT_BULK.bigmodel)
    real_direct_dir = os.path.join(input_param.root_dir, INIT_BULK.direct)
     
    result_save_path = []
    for init_config in init_configs:
        init_config_name = "init_config_{}".format(init_config.config_index)
        
        collection_work_dir = os.path.join(collection_dir, init_config_name)
        if not os.path.exists(collection_work_dir):
            os.makedirs(collection_work_dir)
        #1. copy relax config file
        if init_config.relax:
            source_config = os.path.join(relax_dir, init_config_name, DFT_STYLE.get_relaxed_config(resource.dft_style))
            target_config = os.path.join(collection_work_dir, DFT_STYLE.get_relaxed_config(resource.dft_style))
            copy_file(source_config, target_config)
        #2. copy super cell and scaled config file
        config_template = "*{}".format(DFT_STYLE.get_postfix(resource.dft_style))
        source_file_list = search_files(os.path.join(duplicate_scale_dir, init_config_name), config_template)
        for file in source_file_list:
            target_file = os.path.join(collection_dir, init_config_name, os.path.basename(file))
            copy_file(file, target_file)
                    
        #3. copy perturb structure
        if init_config.perturb is not None:
            source_dir = search_files(os.path.join(pertub_dir, init_config_name), "*")
            for source in source_dir:
                # copy .../path/pertub/init_config_0/0.95_scale to .../path/collection/init_config_0/0.95_scale
                target_dir = os.path.join(collection_dir, init_config_name, "{}_{}".format(os.path.basename(source),INIT_BULK.pertub))
                copy_dir(source, target_dir, symlinks=False)
            
        #4. copy aimd result
        source_aimd = []
        if init_config.aimd is True:
            source_aimd = search_files(os.path.join(aimd_dir, init_config_name), "*/*{}/{}".format(INIT_BULK.aimd, DFT_STYLE.get_aimd_config(resource.dft_style)))
            if len(source_aimd) == 0:
                continue
            source_aimd = sorted(source_aimd)
            #5. convert the aimd files (for vasp is outcar, for pwmat is movement) to npy format
            extract_pwdata(input_data_list=source_aimd, 
                            intput_data_format= DFT_STYLE.get_aimd_config_format(resource.dft_style),
                            save_data_path  = os.path.join(collection_dir, init_config_name),
                            save_data_name  = INIT_BULK.get_save_format(input_param.data_format), 
                            save_data_format= input_param.data_format,
                            data_shuffle=input_param.data_shuffle, 
                            interval=1
                )
            result_save_path.append(os.path.join(real_collection_dir, init_config_name, INIT_BULK.get_save_format(input_param.data_format)))
                
    #6 convert relabel datas
    if input_param.is_scf:
        source_scf = search_files(relabel_dir,\
                "*/{}".format(DFT_STYLE.get_scf_config(resource.dft_style)))
                # init/0-aimd/0-scf/OUTCAR
        source_scf = sorted(source_scf, key=lambda x:int(os.path.basename(os.path.dirname(x))), reverse=False)
        #5. convert the aimd files (for vasp is outcar, for pwmat is movement) to npy format
        extract_pwdata(input_data_list=source_scf, 
                intput_data_format= DFT_STYLE.get_format_by_postfix(os.path.basename(source_scf[0])),
                save_data_path  = os.path.join(collection_dir, INIT_BULK.scf), 
                save_data_name  = INIT_BULK.get_save_format(input_param.data_format), 
                save_data_format= input_param.data_format,
                data_shuffle=input_param.data_shuffle, 
                interval=1
            )
        result_save_path.append(os.path.join(real_collection_dir, INIT_BULK.scf, INIT_BULK.get_save_format(input_param.data_format)))
    #7 bigmodel infos
    if input_param.is_bigmodel:
        if not input_param.is_scf:
            extract_pwdata(input_data_list=[os.path.join(direct_dir, INIT_BULK.direct_traj)], 
                    intput_data_format= PWDATA.extxyz,
                    save_data_path  = direct_dir, 
                    save_data_name  = INIT_BULK.get_save_format(input_param.data_format), 
                    save_data_format= input_param.data_format,
                    data_shuffle=input_param.data_shuffle, 
                    interval=1
                )
            result_save_path.append(os.path.join(real_collection_dir, INIT_BULK.bigmodel, INIT_BULK.direct, INIT_BULK.get_save_format(input_param.data_format)))
        # copy bigmodel and direct files to realdir
        copy_dir(bigmodel_dir, os.path.join(collection_dir, INIT_BULK.bigmodel))        
    
    if len(result_save_path) > 0:
        result_lines = ["\"{}\",".format(_) for _ in result_save_path]
        result_lines = "\n".join(result_lines)
        # result_lines = result_lines[:-1] # Filter the last ','
        result_save_path = os.path.join(collection_dir, INIT_BULK.npy_format_name)
        write_to_file(result_save_path, result_lines, mode='w')

    # delete link files
    del_file(real_relax_dir)
    del_file(real_duplicate_scale_dir)
    del_file(real_pertub_dir)
    del_file(real_aimd_dir)
    del_file(real_relabel_dir)
    del_file(real_bigmodel_dir)
    # del slurm logs and tags
    del_file_list_by_patten(os.path.join(collection_dir, INIT_BULK.bigmodel), "slurm-*")
    del_file_list_by_patten(os.path.join(collection_dir, INIT_BULK.bigmodel, INIT_BULK.direct), "slurm-*")
    # copy collection file to target
    copy_dir(collection_dir, real_collection_dir)
    if not input_param.reserve_work:
        # mv collection to real dir
        temp_work_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir)
        del_file_list([temp_work_dir])


def scancel_jobs(work_dir):
    relax_job = os.path.join(work_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.relax)
    scf_job = os.path.join(work_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.aimd)
    print("Cancel aimd task:")
    scancle_job(scf_job)
    print("Cancel relax task:")
    scancle_job(relax_job)


"""
    realx dor strcuture
    --/work_path/
    ------------/relax
    -----------------/*_init_config/*_init_.config, etot.input, pseudo files, final.config, mvms_relax
    ------------/super_cell_scale
    -----------------/*_init_config/scale.config, from final.config or *_init_.config
    ------------/perturb
    --------------------/*_init_config/scale.config or final.config
    ----------------------------------/structures/*.config_perturb
    ------------/AIMD
    -----------------/*_init_config/*_config_dir the config file from pertub or *_init_.config
    -------------------------------/etot.input, atom.config, ...
    -----------/result
    -----------------/*_init_config/relax_mvm_*_init_config if need; aimd_mvm_*_init_config if need
    
"""
import os

from pwact.active_learning.user_input.resource import Resource
from pwact.active_learning.user_input.init_bulk_input import InitBulkParam
from pwact.active_learning.init_bulk.duplicate_scale import get_config_files_with_order

from pwact.utils.constant import PWMAT, INIT_BULK, TEMP_STRUCTURE, SLURM_OUT, DFT_STYLE
from pwact.active_learning.slurm.slurm import SlurmJob, Mission
from pwact.utils.slurm_script import get_slurm_job_run_info, split_job_for_group, set_slurm_script_content
    
from pwact.utils.file_operation import write_to_file, link_file, del_file_list_by_patten
from pwact.utils.app_lib.common import link_pseudo_by_atom, set_input_script
from pwact.data_format.configop import save_config, get_atom_type

class AIMD(object):
    def __init__(self, resource: Resource, input_param:InitBulkParam):
        self.resource = resource
        self.input_param = input_param
        self.init_configs = self.input_param.sys_config
        self.relax_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.relax)
        self.super_cell_scale_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.super_cell_scale)
        self.pertub_dir = os.path.join(self.input_param.root_dir,TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.pertub)
        self.aimd_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.aimd)
        self.real_aimd_dir = os.path.join(self.input_param.root_dir, INIT_BULK.aimd)
        
    def make_aimd_work(self):
        aimd_paths = []
        use_dftb = False
        for init_config in self.init_configs:
            if init_config.aimd is False:
                continue
            init_config_name = "init_config_{}".format(init_config.config_index)
            # config_list, config_type = get_config_files_with_order(self.super_cell_scale_dir, self.relax_dir, init_config_name, init_config.config, self.pertub_dir)
            config_list, config_type = get_config_files_with_order(
            super_cell_scale_dir=self.super_cell_scale_dir,
            relax_dir=self.relax_dir,
            init_config_dirname=init_config_name, 
            init_config_path=init_config.config_file, 
            pertub_dir=self.pertub_dir,
            dft_style=self.resource.dft_style
            )

            for index, config in enumerate(config_list):
                if config_type == INIT_BULK.pertub:
                    tmp_config_dir = os.path.basename(os.path.basename(os.path.dirname(config)))
                elif config_type == INIT_BULK.super_cell or config_type == INIT_BULK.scale:
                    tmp_config_dir = os.path.basename(config).replace(DFT_STYLE.get_postfix(self.resource.dft_style), "")
                elif config_type == INIT_BULK.relax:
                    tmp_config_dir = INIT_BULK.relax
                else:
                    tmp_config_dir = INIT_BULK.init
                aimd_dir = os.path.join(self.aimd_dir, init_config_name, tmp_config_dir, "{}_{}".format(index, INIT_BULK.aimd))
                if not os.path.exists(aimd_dir):
                    os.makedirs(aimd_dir)
                self.make_aimd_file(
                    aimd_dir=aimd_dir, 
                    config_file=config, 
                    conifg_format=DFT_STYLE.get_format_by_postfix(os.path.basename(config)), 
                    target_format=DFT_STYLE.get_pwdata_format(self.resource.dft_style, is_cp2k_coord=True),
                    input_file=init_config.aimd_input_file, 
                    kspacing=init_config.aimd_kspacing, 
                    flag_symm=init_config.aimd_flag_symm,
                    is_dftb = init_config.use_dftb,
                    in_skf=self.input_param.dft_input.in_skf)

                aimd_paths.append(aimd_dir)
                use_dftb = init_config.use_dftb
        # make slurm script and slurm job
        self.make_aimd_slurm_job_files(aimd_paths, use_dftb)
    
    def check_work_done(self):
        slurm_remain, slurm_success = get_slurm_job_run_info(self.aimd_dir, \
            job_patten="*-{}".format(INIT_BULK.aimd_job), \
            tag_patten="*-{}".format(INIT_BULK.aimd_tag))
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False # len(slurm_remain) > 0 exist slurm jobs need to do
        return slurm_done
            
    def do_aimd_jobs(self):
        mission = Mission()
        slurm_remain, slurm_success = get_slurm_job_run_info(self.aimd_dir, \
            job_patten="*-{}".format(INIT_BULK.aimd_job), \
            tag_patten="*-{}".format(INIT_BULK.aimd_tag))
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False # len(slurm_remain) > 0 exist slurm jobs need to do
        if slurm_done is False:
            #recover slurm jobs
            if len(slurm_remain) > 0:
                print("Run these aimd Jobs:\n")
                print(slurm_remain)
                for i, script_path in enumerate(slurm_remain):
                    slurm_job = SlurmJob()
                    tag_name = "{}-{}".format(os.path.basename(script_path).split('-')[0].strip(), INIT_BULK.aimd_tag)
                    tag = os.path.join(os.path.dirname(script_path),tag_name)
                    slurm_job.set_tag(tag)
                    slurm_job.set_cmd(script_path)
                    mission.add_job(slurm_job)

            if len(mission.job_list) > 0:
                mission.commit_jobs()
                mission.check_running_job()
                mission.all_job_finished(error_type=SLURM_OUT.dft_out)
                # mission.move_slurm_log_to_slurm_work_dir()

    '''
    description: 
        input_file is aimd input control file, for vasp is incar, for pwmat is etot.input
    return {*}
    author: wuxingxing
    '''    
    def make_aimd_file(self, aimd_dir:str, config_file:str, conifg_format:str, target_format:str, \
                input_file:str, kspacing:float=None, flag_symm:int=None, is_dftb:bool=False, in_skf:str=None):
        #1. set config file
        target_config = save_config(config=config_file, 
                                    input_format=conifg_format,
                                    wrap = False, 
                                    direct = True, 
                                    sort = True, 
                                    save_format=DFT_STYLE.get_pwdata_format(dft_style=self.resource.dft_style, is_cp2k_coord=True), 
                                    save_path=aimd_dir, 
                                    save_name=DFT_STYLE.get_normal_config(self.resource.dft_style))

        atom_type_list, _ = get_atom_type(config_file, conifg_format)
        #2. set pseudo files
        if not is_dftb:
            pseudo_names = link_pseudo_by_atom(
                pseudo_list     = self.input_param.dft_input.pseudo, 
                target_dir      = aimd_dir, 
                atom_order      = atom_type_list, 
                dft_style       = self.resource.dft_style,
                basis_set_file  =self.input_param.dft_input.basis_set_file,
                potential_file  =self.input_param.dft_input.potential_file
                )
        else:
            # link in.skf path to aimd dir
            pseudo_names = []
            target_dir = os.path.join(aimd_dir, PWMAT.in_skf)
            if in_skf is not None:
                link_file(in_skf, target_dir)
        #3. make dft input file
        set_input_script(
            input_file=input_file,
            config=target_config,
            dft_style=self.resource.dft_style,
            kspacing=kspacing, 
            flag_symm=flag_symm, 
            save_dir = aimd_dir,
            pseudo_names=pseudo_names,
            gaussian_base_param = self.input_param.dft_input.gaussian_base_param
        )

    def make_aimd_slurm_job_files(self, aimd_dir_list:list[str],use_dftb: bool=False):
        del_file_list_by_patten(self.aimd_dir, "*{}".format(INIT_BULK.aimd_job))
        group_list = split_job_for_group(self.resource.dft_resource.group_size, aimd_dir_list, self.resource.dft_resource.parallel_num)
        for group_index, group in enumerate(group_list):
            if group[0] == "NONE":
                continue
            jobname = "aimd{}".format(group_index)
            tag_name = "{}-{}".format(group_index, INIT_BULK.aimd_tag)
            tag = os.path.join(self.aimd_dir, tag_name)
            run_cmd = self.resource.dft_resource.command
            group_slurm_script = set_slurm_script_content(gpu_per_node=self.resource.dft_resource.gpu_per_node, 
                number_node = self.resource.dft_resource.number_node, 
                cpu_per_node = self.resource.dft_resource.cpu_per_node,
                queue_name = self.resource.dft_resource.queue_name,
                custom_flags = self.resource.dft_resource.custom_flags,
                env_script = self.resource.dft_resource.env_script,
                job_name = jobname,
                run_cmd_template = run_cmd,
                group = group,
                job_tag = tag,
                task_tag = INIT_BULK.aimd_tag, 
                task_tag_faild = INIT_BULK.aimd_tag_failed,
                parallel_num=self.resource.dft_resource.parallel_num,
                check_type=self.resource.dft_style
                )
            slurm_script_name = "{}-{}".format(group_index, INIT_BULK.aimd_job)
            slurm_job_file =  os.path.join(self.aimd_dir, slurm_script_name)
            write_to_file(slurm_job_file, group_slurm_script, "w")

    def do_post_process(self):
        if os.path.exists(self.aimd_dir):
            link_file(self.aimd_dir, self.real_aimd_dir)



import os
import numpy as np
import subprocess
from pwact.utils.constant import PWMAT, VASP, PWDATA, get_atomic_name_from_number
from pwact.utils.file_operation import del_file, copy_file
from pwdata import Config
# '''
# description: 
#     lammps dump file to poscar format or pwmat format
# param {str} dump_file
# param {str} save_file
# param {list} type_map
# param {bool} unwrap
# return {*}
# author: wuxingxing
# '''
# def lammps_dump_to_config(dump_file:str, save_file:str, type_map:list[str], unwrap:bool=False, dft_style:str=None):
#     system = dp_system(dump_file, type_map=type_map, begin=0, step=1, unwrap=False, fmt = 'lammps/dump')
#     tmp_poscar = os.path.join(os.path.dirname(save_file), "tmp_psocar")
#     system.to_poscar(tmp_poscar, frame_idx=0)
#     if dft_style == DFT_STYLE.pwmat:
#         target_file = poscar_to_atom_config(
#             poscar=tmp_poscar, 
#             save_dir=os.path.dirname(save_file), 
#             save_name=os.path.basename(save_file)
#             )
#         del_file(tmp_poscar)
#     elif dft_style == DFT_STYLE.vasp:
#         target_file = os.path.join(os.path.dirname(save_file), VASP.poscar)
#         copy_file(tmp_poscar, target_file)
#         del_file(tmp_poscar)
#     return target_file
        


'''
description: 
    1. copy the source poscar to save_dir/tmp_poscar
    2. convert the tmp_poscar to atom.config
    3. change the atom.config file name to save_name
    4. delete the tmp_poscar file
param {str} poscar
param {str} save_dir
param {str} save_name
return {*}
author: wuxingxing
'''
def poscar_to_atom_config(poscar:str, save_dir:str, save_name:str=None):
    tmp_poscar = os.path.join(save_dir, "tmp_poscar")
    copy_file(poscar, tmp_poscar)
    cwd = os.getcwd()
    os.chdir(save_dir)
    subprocess.run(["poscar2config.x {} > /dev/null".format(os.path.basename(tmp_poscar))], shell = True)
    if save_name is not None:
        subprocess.run(["mv {} {} > /dev/null".format(PWMAT.atom_config, save_name)], shell = True)
        target_file = os.path.join(save_dir, save_name)
    else:
        target_file = os.path.join(save_dir, PWMAT.atom_config)
    os.chdir(cwd)
    del_file(tmp_poscar)
    return target_file

'''
description:
    1. copy the source atom.config to save_dir/tmp_atom.config
    2. convert the tmp_atom.config to POSCAR
    3. change the POSCAR file name to save_name
    4. delete the tmp_atom.config file
param {str} atom_config
param {str} save_dir
param {str} save_name
return {*}
author: wuxingxing
'''
def atom_config_to_poscar(atom_config:str, save_dir:str, save_name:str=None):
    tmp_atom_config = os.path.join(save_dir, "tmp_atom.config")
    copy_file(atom_config, tmp_atom_config)
    cwd = os.getcwd()
    os.chdir(save_dir)
    subprocess.run(["config2poscar.x {} > /dev/null".format(os.path.basename(tmp_atom_config))], shell = True)
    # remove the black space in ' Selective Dynamics' line in POSCAR file
    subprocess.run(["sed -i '/^ *Selective Dynamics/s/^ *//' {} > /dev/null".format(VASP.poscar)], shell = True)
    # remame the poscar file
    if save_name is not None:
        subprocess.run(["mv {} {} > /dev/null".format(VASP.poscar, save_name)], shell = True)
        target_file = os.path.join(save_dir, save_name)
    else:
        target_file = os.path.join(save_dir, VASP.poscar)
    os.chdir(cwd)
    del_file(tmp_atom_config)# delete temp file
    return target_file

def _reciprocal_box(box):
    rbox = np.linalg.inv(box)
    rbox = rbox.T
    return rbox

# def _make_pwmat_kp_mp(kpoints):
#     ret = ""
#     ret += "%d %d %d 0 0 0 " % (kpoints[0], kpoints[1], kpoints[2])
#     return ret

def _make_kspacing_kpoints(config, kspacing):
    config = Config(data_path=config, format="pwmat/config")
    lattice = config.images[0].lattice
    # box = []
    # for idx, ii in enumerate(lines):
    #     if "LATTICE" in ii.upper():
    #         for kk in range(idx + 1, idx + 1 + 3):
    #             vector = [float(jj) for jj in lines[kk].split()[0:3]]
    #             box.append(vector)
    #         box = np.array(box)
    #         rbox = _reciprocal_box(box)
    rbox = _reciprocal_box(lattice)
    kpoints = [
        round(2 * np.pi * np.linalg.norm(ii) / kspacing) for ii in rbox
    ]
    kpoints[0] = 1 if kpoints[0] == 0 else kpoints[0]
    kpoints[1] = 1 if kpoints[1] == 0 else kpoints[1]
    kpoints[2] = 1 if kpoints[2] == 0 else kpoints[2]
    ret = ""
    ret += "%d %d %d 0 0 0 " % (kpoints[0], kpoints[1], kpoints[2])
    return ret
    # ret = _make_pwmat_kp_mp(kpoints)

def make_pwmat_input_dict(
    node1,
    node2,
    job_type,
    atom_config,
    pseudo_list,
    ecut,
    ecut2,
    e_error,
    rho_error,
    out_force,
    energy_decomp,
    out_stress,
    icmix=None,
    smearing=None,
    sigma=None,
    kspacing=0.5,
    flag_symm=None,
    out_wg="F",
    out_rho="F",
    out_mlmd="T",
    relax_detail=None,
    vdw = None
):
    icmix, smearing, sigma = _make_smearing(icmix, smearing, sigma)
    flag_symm = _make_flag_symm(flag_symm)
        
    script = ""
    script += "{} {}\n".format(node1, node2)
    script += "job = {}\n".format(job_type)
    script += "{} = {}\n".format("in.atom", os.path.basename(atom_config))
    for pse_index, pseudo in enumerate(pseudo_list):
        script += "IN.PSP{} = {}\n".format(pse_index+1, pseudo)
    script += "{} =  {}\n".format("ecut", ecut)
    script += "{} = {}\n".format("ecut2", ecut2)
    script += "{} = {}\n".format("e_error", e_error)
    script += "{} = {}\n".format("rho_error", rho_error)
    script += "{} = {}\n".format("out.force", out_force)
    script += "{} = {}\n".format("energy_decomp", energy_decomp)
    script += "{} = {}\n".format("out.stress", out_stress)
    if icmix is not None:
        if sigma is not None:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " " + str(smearing)
                SCF_ITER0_2 = (
                    "94 4 3 " + str(icmix) + " " + str(sigma) + " " + str(smearing)
                )
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " 2"
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " " + str(sigma) + " 2"

        else:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 " + str(smearing)
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " 0.025 " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 2"
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " 0.025 2"
    else:
        if sigma is not None:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " " + str(smearing)
                SCF_ITER0_2 = "94 4 3 1.0000 " + str(sigma) + " " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " 2"
                SCF_ITER0_2 = "94 4 3 1.0000 " + str(sigma) + " 2"
        else:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 " + str(smearing)
                SCF_ITER0_2 = "94 4 3 1.0000 0.025 " + str(smearing)
            else:#run this config
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 2"
                SCF_ITER0_2 = "94 4 3 1.0000 0.025 2"
    script += "{} = {}\n".format("scf_iter0_1", SCF_ITER0_1)
    script += "{} = {}\n".format("scf_iter0_2", SCF_ITER0_2)
    if flag_symm is not None:
        MP_N123 = _make_kspacing_kpoints(atom_config, kspacing)
        MP_N123 += str(flag_symm)
    else:
        MP_N123 = _make_kspacing_kpoints(atom_config, kspacing)
    # use pwmat defualt value
    script += "{} = {}\n".format("mp_n123", MP_N123)
    script += "{} = {}\n".format("out.wg", out_wg)
    script += "{} = {}\n".format("out.rho", out_rho)
    script += "{} = {}\n".format("out.mlmd", out_mlmd)
    # if do relax
    if job_type == PWMAT.relax:
        if relax_detail is not None:
            script += "{} = {}\n".format("relax_detail", relax_detail)
        if vdw is not None:
            script += "{} = {}\n".format("vdw", vdw)
    return script

def _make_smearing(icmix=None, smearing = None, sigma = None):
    if icmix is None:
        if smearing is None:
            if sigma is None:
                return None, None, None
            else:
                return None, None, sigma
        else:
            if sigma is None:
                return None, smearing, None
            else:
                return None, smearing, sigma
    else:
        if smearing is None:
            if sigma is None:
                return icmix, None, None
            else:
                return icmix, None, sigma
        else:
            if sigma is None:
                return icmix, smearing, None
            else:
                return icmix, smearing, sigma
    

def _make_flag_symm(flag_symm = None):
    if flag_symm is None:
        return None
    if flag_symm == "NONE":
        flag_symm = None
    elif str(flag_symm) not in [None, "0", "1", "2", "3"]:
        raise RuntimeError("unknow flag_symm type " + str(flag_symm))
    return flag_symm

def read_and_check_etot_input(etot_input_path:str):
    with open(etot_input_path, "r") as fp:
        lines = fp.readlines()
    
    etot_lines = lines
    # key_type = ["string_keys", "char_keys", "bool_keys", "int_keys", "float_keys"]
    key_values = {}
    for i in etot_lines[1:]:
        i = i.strip()
        if "#" in i[:2]:
            continue
        key = i.split('=')[0].strip().upper()
        value = None
        if key in string_keys:
            value = i.split('=')[1]
        if key in char_keys:
            value = i.split('=')[1:]
            if len(value) > 1:
                raise Exception(" {} error, value should be char type, please check the file {}!".format(i, etot_input_path))
        elif key in bool_keys:#USE_DFTB
            value = i.split('=')[1].strip().upper()
            if key == "ENERGY_DECOMP":
                pass
            elif value not in ["T", "F"]:
                raise Exception(" {} error, value should be T or F, please check the file {}!".format(i, etot_input_path))
        elif key in int_keys:
            try:
                value = int(i.split('=')[1].strip().upper())
            except Exception:
                raise Exception(" {} error, value should be int type, please check the file {}!".format(i, etot_input_path))
        elif key in float_keys:
            try:
                value = float(i.split('=')[1].strip().upper())
            except Exception:
                raise Exception(" {} error, value should be float type, please check the file {}!".format(i, etot_input_path))
        key_values[key] = value
    return key_values, etot_lines

'''
description: 
    check mp_123 and kspacing for scf etot.input file
param {*} self
param {str} etot_input
param {None} kspacing
return {*}
author: wuxingxing
'''
def check_kspacing_mp_123(etot_input:str, kspacing:None):
    key_values, etot_lines = read_and_check_etot_input(etot_input)
    if kspacing is not None and "MP_123" in key_values:
        error_info = "The 'kspacing' in DFT/input/{} dict and 'MP_123' in ethot.input cannot coexist.\n".format(os.path.basename(etot_input))
        error_info += "If 'MP_123' is not indicated in DFT/input/{}, we will use 'kspacing' param to generate the 'MP_123' parameter\n".format(os.path.basename(etot_input))
        raise Exception(error_info)
        

'''
description: 
    set etot.input content, 
    if the etot_input_file exists, read and check format, then supplement unset parameters
    else set the content according to user input json file
    
param {str} etot_input_file
param {str} atom_config
return {*}
author: wuxingxing
'''
def set_etot_input_by_file(
    etot_input_file:str, 
    kspacing:float=None, 
    flag_symm:int=None, 
    atom_config:str=None, 
    pseudo_names:list[str]=None,
    is_scf = False, # if True, job is scf, and 'OUT.MLMD = T' to etot.input
    is_skf_file = False,  # if True, set in.skf to etot.input file
    gaussian_base_param:dict=None
    ):
    key_values, etot_lines = read_and_check_etot_input(etot_input_file)
    is_gaussian = False
    if "USE_GAUSSIAN" in key_values.keys() and key_values["USE_GAUSSIAN"] is not None and key_values["USE_GAUSSIAN"] == "T":
        is_gaussian = True
    
    is_skf = False
    if "USE_DFTB" in key_values.keys() and key_values["USE_DFTB"] is not None and key_values["USE_DFTB"] == "T":
        if key_values["DFTB_DETAIL"].replace(",", " ").split()[0] != "3": # not chardb
            is_skf = True
    
    index = 0
    new_etot_lines = []
    while index < len(etot_lines):
        # remove the in.atom, in.skf in.psp* in etot.input file if exists
        if "in.atom".upper() in etot_lines[index].upper():
            # etot_lines[index].remove(etot_lines[index])
            pass
        elif "in.skf".upper() in etot_lines[index].upper():
            # etot_lines[index].remove(etot_lines[index])
            pass
        elif "IN.PSP" in etot_lines[index].upper(): # to avoid the new_etot_lines add 'IN.PSP' and 'in.skf' 'in.atom' in etot_lines
            pass
        elif "IN.BASIS" in etot_lines[index].upper(): # to avoid the new_etot_lines add 'IN.BASIS' in etot_lines
            pass
            # etot_lines.remove(etot_lines[index])
        else:
            new_etot_lines.append(etot_lines[index])
        index += 1
    new_etot_lines.append("\nIN.ATOM = {}\n".format(os.path.basename(atom_config)))
    atom_type_numbers = Config(format=PWDATA.pwmat_config, data_path=atom_config).images[0].atom_type
    atom_type_names = get_atomic_name_from_number(atom_type_numbers)
    # if dftb and need in_skf
    if is_skf and is_skf_file:
        new_etot_lines.append("IN.SKF = ./{}/\n".format(PWMAT.in_skf))
    # is not for dftb, reset the IN.PSP
    if "USE_DFTB" not in key_values.keys() or key_values["USE_DFTB"] is None and key_values["USE_DFTB"] == "F":
        if is_gaussian is False:
            for pseudo_i, pseudo in enumerate(pseudo_names):
                new_etot_lines.append("IN.PSP{} = {}\n".format(pseudo_i + 1, pseudo))
        else: # pwmat gaussian
            psp_line = set_gassion_psp(atom_type_names, gaussian_base_param)
            new_etot_lines.append(psp_line)
    key_list = list(key_values)
    # set OUT.MLMD
    if "OUT.MLMD" not in key_list:
        if is_scf:
            new_etot_lines.append("OUT.MLMD = T\n")
    # # set OUT.WG OUT.RHO OUT.VR
    if "OUT.WG" not in key_list:
        new_etot_lines.append("OUT.WG = F\n")
    if "OUT.RHO" not in key_list:
        new_etot_lines.append("OUT.RHO = F\n")
    if "OUT.VR" not in key_list:
        new_etot_lines.append("OUT.VR = F\n")
    # if MP_N123 is not in etot.input file then using 'kespacing' generates it
    if "MP_N123" not in key_list:
        kspacing = PWMAT.kspacing_default if kspacing is None else kspacing
        MP_N123 = _make_kspacing_kpoints(atom_config, kspacing)
        if "FLAG_SYMM" in key_list:
            MP_N123 += str(key_values["FLAG_SYMM"])
        else:
            MP_N123 += str(flag_symm)
        new_etot_lines.append("MP_N123 = {}\n".format(MP_N123))

    new_etot_lines.append("\n")
    
    return "".join(new_etot_lines)

def set_gassion_psp(atom_list, gaussian_base_param:dict):
    psp_num = 1
    psp_line = "\n"
    base_line = "\n"
    for idx, atom in enumerate(gaussian_base_param["ELEMENT"]):
        if atom not in atom_list:
            continue
        psp_line  += "IN.PSP{}   = {} {} {}\n".format(psp_num, atom, gaussian_base_param["POTENTIAL"][idx],  gaussian_base_param["POTENTIAL_FILE_NAME"])
        base_line += "IN.BASIS{} = {} {} {}\n".format(psp_num, atom, gaussian_base_param["BASIS_SET"][idx],gaussian_base_param["BASIS_SET_FILE_NAME"])
        psp_num += 1
    psp_line += base_line
    return psp_line

       
def is_alive_atomic_energy(movement_list:list):
    if len(movement_list) < 1:
        return False
    # Declare is_real_Ep as a global variable
    command = 'grep Atomic-Energy ' + movement_list[0] + ' | head -n 1'
    print('running-shell-command: ' + command)
    result = subprocess.run(command, stdout=subprocess.PIPE, encoding='utf-8', shell=True)
    if 'Atomic-Energy' in result.stdout:
        alive_atomic_energy = True
    else:
        alive_atomic_energy = False
    return alive_atomic_energy
    
'''
Author: WuXing wuxingxing
Date: 2023-02-28 16:48:49
'''
float_keys=[
    'Ecut',
    'Ecut2',
    'Ecut2L',
    'EcutP',
    'HSE_OMEGA',
    'HSE_ALPHA',
    'HSE_BETA',
    'NUM_ELECTRON',
    'WG_ERROR',
    'E_ERROR',
    'RHO_ERROR',
    'FERMIDE',
    'KSPACING',
    "SYMM_PREC" #DFTB param
    ]

int_keys=[
    'SPIN',
    'CONSTRAINT_MAG',
    'SPIN222_MAGDIR_STEPFIX',
    'DFTD3_VERSION',
    'NUM_BAND',
    'FLAG_SYMM',
    "NUM_GPU", #DFTB param
    "RANDOM_SEED"#DFTB param
    ]

bool_keys=[
    'OUT.WG',
    'IN.RHO',
    'OUT.RHO',
    'IN.VR',
    'OUT.VR',
    'IN.VEXT',
    'OUT.VATOM',
    'OUT.FORCE',
    'OUT.STRESS',
    'IN.SYMM',
    'CHARGE_DECOMP',
    'ENERGY_DECOMP',
    'IN.SOLVENT',
    'IN.NONSCF',
    'IN.OCC',
    'IN.OCC_ADIA',
    'OUT.MLMD',
    'NUM_BLOCKED_PSI',
    'OUT.RHOATOM',
    "USE_DFTB",
    "USE_GAUSSIAN"
    ]

char_keys=['PRECISION',
           'JOB',
           'IN.ATOM',
           'CONVERGENCE',
           'ACCURACY',
           'XCFUNCTIONAL',
           'VDW']

string_keys=['VFF_DETAIL',
           'EGG_DETAIL',
           'N123',
           'NS123',
           'N123L',
           'P123',
           'MP_N123',
           'NQ123',
           'RELAX_DETAIL',
           'MD_DETAIL',
           "DFTB_DETAIL",#DFTB param
           'MD_SPECIAL',
           'SCF_SPECIAL',
           'STRESS_CORR',
           'HSE_DETAIL',
           'RELAX_HSE',
           'IN.OCC',
           'SCF_ITER0_1',
           'SCF_ITER0_2',
           'SCF_ITER1_1',
           'ENERGY_DECOMP_SPECIAL',
           'ENERGY_DECOMP_SPECIAL1',
           'ENERGY_DECOMP_SPECIAL2',
           'ENERGY_DECOMP_COULOMB',
           ]



import subprocess
import pkg_resources
def check_envs():
    # for pwmat
    comm_info()

def comm_info():
    print("\n" + "=" * 50) 
    print("         PWACT Basic Information")
    print("=" * 50) 
    print("Version: 0.3.1")
    print("Compatible pwdata: >= 0.5.0")
    print("Compatible MatPL: >= 2025.3")
    print("Contact: support@pwmat.com")
    print("Citation: https://github.com/LonxunQuantum/MatPL")
    print("Manual online: http://doc.lonxun.com/PWMLFF/")
    print("=" * 50)  
    print("\n\n")



import numpy as np
import argparse
import os
import shutil
import glob
import json
from pwdata.config import Config
def make_kspacing_kpoints(config, format, kspacing):
    config = Config(format=format, data_path=config)
    # with open(config, "r") as fp:
    #     lines = fp.read().split("\n")
    # box = []
    # for idx, ii in enumerate(lines):
    #     if "LATTICE" in ii.upper():
    #         for kk in range(idx + 1, idx + 1 + 3):
    #             vector = [float(jj) for jj in lines[kk].split()[0:3]]
    #             box.append(vector)
    #         box = np.array(box)
    #         rbox = _reciprocal_box(box)
    box = config.images[0].lattice
    rbox = _reciprocal_box(box)
    kpoints = [
        max(1, round(2 * np.pi * np.linalg.norm(ii) / kspacing)) for ii in rbox
    ]
    #DP pwmat multi kpoints will be slow, use round not ceil
    # kpoints = [
    #     max(1, (np.ceil(2 * np.pi * np.linalg.norm(ii) / ks).astype(int)))
    #     for ii, ks in zip(rbox, [kspacing,kspacing,kspacing])
    # ]
    ret = ""
    ret += "%d %d %d 0 0 0 " % (kpoints[0], kpoints[1], kpoints[2])
    print(ret)
    return ret
    
def _reciprocal_box(box):
    rbox = np.linalg.inv(box)
    rbox = rbox.T
    return rbox

def get_energy_dftb_vasp():
    aimd_dir = "/data/home/wuxingxing/datas/al_dir/HfO2/dftb/init_bulk_hfo2/temp_init_bulk_work/aimd"#/init_config_0/init/0_aimd
    scf_dir = "/data/home/wuxingxing/datas/al_dir/HfO2/dftb/init_bulk_hfo2/temp_init_bulk_work/scf"#init_config_0/init/0_aimd/0-scf
    save_file = "/data/home/wuxingxing/datas/al_dir/HfO2/dftb/init_bulk_hfo2/energy_count_xtb.txt"
    aimd_dir = glob.glob(os.path.join(aimd_dir, "init_config_*"))
    aimd_dir = sorted(aimd_dir, key=lambda x: int(os.path.basename(x).split('_')[-1]))

    save_text = []
    for aimd in aimd_dir:
        mvm_config = Config(format="pwmat/movement", data_path=os.path.join(aimd, "init/0_aimd/MOVEMENT"))
        scf_list = glob.glob(os.path.join(scf_dir, os.path.basename(aimd), "init/0_aimd/*-scf"))
        scf_list = sorted(scf_list, key=lambda x: int(os.path.basename(x).split('-')[0]))
        for scf in scf_list:
            index = int(os.path.basename(scf).split('-')[0])
            scf_config = Config(format="vasp/outcar", data_path=os.path.join(scf, "OUTCAR"))
            if index == 0:
                base = scf_config.images[0].Ep - mvm_config.images[index].Ep
            save_text.append("aimd {} index {} dftb_energy {} vasp_energy {} vasp_just {}"\
                .format(os.path.basename(aimd), index, \
                    mvm_config.images[index].Ep, scf_config.images[0].Ep,\
                        scf_config.images[0].Ep - base))
    
    with open(save_file, 'w') as wf:
        for line in save_text:
            wf.write(line)
            wf.write("\n")
if __name__=="__main__":
    # parser = argparse.ArgumentParser()
    # parser.add_argument('-i', '--config', help="specify config file path of config", type=str, default='atom.config')
    # parser.add_argument('-f', '--format', help="specify config file format config", type=str, default='pwmat/config')
    # parser.add_argument('-k', '--kspacing', help="specify the kspacing, the default 0.5", type=float, default=0.5)
    # args = parser.parse_args()
    # make_kspacing_kpoints(config=args.config, format=args.format, kspacing=args.kspacing)
    # make_kspacing_kpoints(config="/data/home/wuxingxing/datas/al_dir/HfO2/dftb/init_bulk_hfo2/temp_init_bulk_work/scf/init_config_0/init/0_aimd/0-scf/POSCAR",
    #  format="vasp/poscar", kspacing=0.5)
    get_energy_dftb_vasp()
